#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <time.h>
#include <math.h>
#include <stdbool.h>
#include <string.h>


#define PICKUP 1
#define DELIVERY 0

#define DIM ((size+1)/2)

/********************************************
 *											*
 *				Dati globali				*
 *											*
 ********************************************/

unsigned int capacity;				/*Capacit del veicolo*/

struct euclidean *position;			/*Puntatore alla posizione delle citt*/

struct data *cities;				/*Contiene i dati di carico e scarico delle citt*/

unsigned int **costMatrix;			/*Matrice dei costi*/

unsigned int size;					/*Numero delle citt*/

unsigned int totalLenght;           /*Lunghezza del tour*/



/*Lista che rappresenta il tour*/

struct list {
	struct list *previous;
	unsigned int city;
	struct list *next;
};



struct euclidean {					/*Posizione delle citt*/
	unsigned int x;
	unsigned int y;
};


struct data {
	unsigned int type;				/*Tipo della citt*/
	unsigned int partner;			/*Citt partner*/
};


struct couples {
    unsigned int city;
    unsigned int partnerPosition;
};


struct list *head;

/********************************************
 *											*
 *		Prototipi delle funzioni			*
 *											*
 ********************************************/
 
/*Legge la posizione delle citt*/
void readPosition(char *file);

/*Legge il matching*/
void readMatching(char *file);

/*Calcola i pesi*/
void weights(void);

/*Legge il tour iniziale*/
void readTour(char *file);

/*Scambia le coppie*/
void coupleExchange(void);

/*Scambia i blocchi*/
void blockExchange(void);

/*Toglie e reinserisce un blocco*/
void removeBlock(void);

/*Toglie e reinserisce una coppia*/
void removeCouple(void);

/*Trova la posizione migliore per l'inserimento*/
unsigned int findBest(unsigned int city, unsigned int listSize, unsigned int *myStart, unsigned int *myEnd, struct list *head);

/*Euristica di tipo complexNeighbourhoodDescent*/
void complexNeighbourhoodDescent(char *file);

/*Euristica Variable Neighbourhood Descent*/
void variableNeighbourhoodDescent(char *file);

/*Esegue l'euristica di tipo complexNeighbourhoodDescent sui vari tour iniziali*/
void complexNeighbourhoodDescentHeuristic(void);

/*Esegue l'euristica di tipo V.N.D. sui vari tour iniziali*/
void vndHeuristic(void);


/********************************************
 *											*
 *					Main					*
 *											*
 ********************************************/



int main(int argc, char *argv[]) {

    clock_t start, finish;
    FILE *fp;
       
    double duration;
    unsigned int minutes;
    unsigned int seconds;
    unsigned int i;
    
    
    

	/*Sistemare argomenti*/
	readPosition("map.tsp");

	/*Creo spazio per l'array cities*/
	cities=malloc(sizeof(struct data)*size);
	
	/*Leggo il matching*/
	readMatching("matching.mtc");

	/*Calcolo i pesi*/
	weights();
	 
    printf("\n\nPerforming complexNeighbourhoodDescent...\n");
    
    start=clock();
    
    /*Euristica di tipo complexNeighbourhoodDescent*/
    complexNeighbourhoodDescentHeuristic();
    
    finish=clock();
	duration = (double)(finish - start) / CLOCKS_PER_SEC;
	minutes=duration/60;
	seconds= ((unsigned int)duration)%60;
	
    /*Scrivo su file il temp*/
    if ((fp=fopen("complexNeighbourhoodDescent.txt", "a"))==NULL) {
        printf("\nCan't open output file!\n");
        exit(1);
    }    
    
    fprintf(fp, "\nDuration of complexNeighbourhoodDescent is %u minutes and %u seconds\n", minutes, seconds); 
    fclose(fp);
    
    
    printf("\nPerforming V.N.D...\n");
    start=clock();    
    /*Euristica di tipo Variable Neighbourhood Descent*/
    vndHeuristic();
                        
    finish=clock();
	duration = (double)(finish - start) / CLOCKS_PER_SEC;
	minutes=duration/60;
	seconds= ((unsigned int)duration)%60;
	
    printf( "Duration of complexNeighbourhoodDescent is %u minutes and %u seconds\n", minutes, seconds); 

    /*Scrivo su file il temp*/
    if ((fp=fopen("variableNeighbourhoodDescent.txt", "a"))==NULL) {
        printf("\nCan't open output file!\n");
        exit(1);
    }    
    
    fprintf(fp, "\nDuration of variableNeighbourhoodDescent is %u minutes and %u seconds\n", minutes, seconds); 
    fclose(fp);
    
    
    /*Libero la memoria occupata sullo heap*/
    free(cities);
    free(position);
    for (i=0; i<size; i++)
        free(costMatrix[i]);
    free(costMatrix);
    for (i=0; i<size-1; i++) {
        free(head->previous);
        head=head->next;
    }
    free(head);            
	
	return 0;

}
 
 
void readPosition(char *file) {

	FILE *fp;
	unsigned int i;
	unsigned int city, x, y;
    char string[50];
    

	/*Apro il file*/
	if ((fp = fopen(file, "r")) == NULL) {
		printf("Can't open input file %s!\n\n\n", file);
		exit(1);
	}

	/*Cerco il numero delle citt*/
    do
        fscanf(fp, "%s", string);
    while (strcmp(string, "DIMENSION")!=0);
      
    /*Leggo il carattere ":"*/
    fscanf(fp, "%s", string);
    
    /*Leggo il numero di citt*/
    fscanf(fp, "%d", &size);


	/*Creo spazio per l'array che contiene le posizioni*/
	position = malloc(sizeof(struct euclidean)*size);

 
    /*Mi posiziono sull'elenco delle coordinate delle citt*/
    do 
        fscanf(fp, "%s", string);
    while (strcmp(string, "NODE_COORD_SECTION")!=0);

	/*Leggo i valori*/
	for (i=0; i<size; i++) {
		fscanf(fp, "%u", &city);
		fscanf(fp, "%u", &x);
		fscanf(fp, "%u", &y);
		position[i].x=x;
		position[i].y=y;
	}

	fclose(fp);


}


void readMatching(char *file) {

	FILE *fp;
	unsigned int i,j;

	/*Apro il file*/
	if ((fp = fopen(file, "r")) == NULL) {
		printf("Can't open input file %s!\n\n\n", file);
		exit(1);
	}

	/*Leggo i dati*/
	for (i=0; i<size; i++) {
		fscanf(fp, "%u %u %u", &j, &cities[i].type, &cities[i].partner);
	}
	fclose(fp);


}



void weights(void) {

	unsigned int i,j;

	int px, py;


	/*Creo spazio per la matrice dei costi*/
	costMatrix = malloc(sizeof(int*)*size);
			
	for (i=0; i<size; i++) {
		costMatrix[i]=malloc(sizeof(int)*size);
    }

	/*Calcolo la distanza tra ogni coppia di citt, arrotondando a intero*/
	for (i=0; i<size; i++)
		for (j=0; j<size; j++) {
			px = position[i].x - position[j].x;
			py = position[i].y - position[j].y;
			costMatrix[i][j] = trunc(sqrt(pow(px, 2.0) + pow(py, 2.0))+0.5);
        }

}

 
void readTour(char *file) {

	FILE *fp;
	unsigned int i, x;
	struct list *current;

	/*Apro il file contenente il tour iniziale*/
	if ((fp=fopen(file, "r"))==NULL) {
		printf("Can't open %s", file);
		exit(1);
	}


	/*Inserisco l'elemento 0 in lista*/
	head=malloc(sizeof(struct list));
	head->previous=NULL;
	head->city=0;
	head->next=NULL;

	current=head;


	/*Leggo il file*/
	for (i=0; i<size+1; i++) {

		/*Memorizzo il nome della citt e il carico parziale*/
		current->next=malloc(sizeof(struct list));
		(current->next)->previous=current;
		current=current->next;
		fscanf(fp, "%u", &x);
		current->city=x;
	}

	/*Chiudo la lista rendendola circolare*/
	current->next=head->next;
	free(head);
	head=current->next;
	head->previous=current;

	/*Leggo la lunghezza del tour*/
	fscanf(fp, "%u", &totalLenght);
    fscanf(fp, "%u", &totalLenght);
        				
	/*Chiudo il file*/
	fclose(fp);
      
}
 
 
 
void coupleExchange(void) {
        
    unsigned int lenght=totalLenght;
    
    struct list *current=head->next;
          
    unsigned int array[size+1];
    
    unsigned int temp[size+1];
           
    unsigned int i, first, second, third, fourth;
    
    unsigned int bestFirst, bestSecond, bestThird, bestFourth;
    
    unsigned int cost, var;
   
    bool improved=true;
    
        
    /*Inizializzo gli array*/
    array[0]=temp[0]=array[size]=temp[size]=0;
    
    for (i=1; i<size; i++) {
        array[i]=temp[i]=current->city;
        current=current->next;
    }
    
             
        improved=false;
        
        /*Posiziono il primo indice*/
        for (first=1; first<size; first++) {
                        
            /*Quando arrivo alla fine mi fermo*/
            if (array[first]==0)
                break;
            
            if (cities[array[first]].type!=PICKUP)
                continue;
            
            second=first+1;
            
            /*Mi posiziono sulla DELIVERY di first*/
            while (array[second]!=cities[array[first]].partner)
                ++second;
            
            /*Mi posiziono su una citt PICKUP*/
            for (third=first+1; third<size; third++) {
            
                if (array[third]==0)
                    break;
                
                if (cities[array[third]].type!=PICKUP)
                    continue;
                    
                fourth=third+1;
                
                /*Mi posiziono sulla DELIVERY di third*/
                while ((array[fourth])!=(cities[array[third]].partner))
                    ++fourth;                
            
                /*Inizializzo temp*/
                for (i=0; i<size+1; i++)
                    temp[i]=array[i];
                
                
                /*Procedo con lo scambio*/
                
                cost=0;
                
                var=temp[first];
                temp[first]=temp[third];
                temp[third]=var;           
            
                var=temp[second];
                temp[second]=temp[fourth];
                temp[fourth]=var;
                
                for (i=0; i<size; i++)
                    cost+=costMatrix[temp[i]][temp[i+1]];
                 
                /*Se lo scambio  conveniente*/                                   
                if (cost<lenght) {
                    bestFirst=first;
                    bestSecond=second;
                    bestThird=third;
                    bestFourth=fourth;
                    lenght=cost;
                    improved=true;
                
                }
                
            
            }
                    
            
        }
        
        /*Se ho migliorato effettuo lo scambio*/
        if (improved) {
            var=array[bestFirst];
            array[bestFirst]=array[bestThird];
            array[bestThird]=var;           
            
            var=array[bestSecond];
            array[bestSecond]=array[bestFourth];
            array[bestFourth]=var;
        }
        
         
   
    
    totalLenght=lenght;
    
    i=1;
    
    /*Aggiorno il tour*/
    for (current=head->next; current!=head; current=current->next)
        current->city=array[i++];
    
}
    
    
void blockExchange(void) {
   
    
    struct list *first, *second, *third, *fourth;
    
    struct list *bestFirst, *bestSecond, *bestThird, *bestFourth;
    
    struct list *current;
    
    unsigned int lenght=totalLenght;
    
    unsigned int cost;
    
    bool near;
    
    bool improved=true;
    
    
   
    
        improved=false;
        
        /*Per tutti gli elementi*/
        for (first=head->next; first->city!=0; first=first->next) {
        
            /*Mi posiziono su una citt PICKUP*/
            if (cities[first->city].type!=PICKUP)
                continue;
        
            /*Mi posiziono sulla DELIVERY relativa*/
            second=first->next;
        
            while (second->city!=cities[first->city].partner)
                second=second->next;
                              
            /*Posiziono third*/
            for (third=second->next; third!=head; third=third->next) {
                        
                /*Mi posiziono su di una citt PICKUP*/
                if (cities[third->city].type!=PICKUP)
                    continue;
                                 
                /*Cerco la DELIVERY corrispondente*/
                fourth=third->next;
                while (fourth->city!=cities[third->city].partner) 
                    fourth=fourth->next;
                
                cost=0;
            
                /*Calcolo il costo dello scambio*/
                 
                /*Se i blocchi NON sono adiacenti*/
                if (second->next!=third) {
                        
                    /*Calcolo i costi fino alla citt precedente first*/
                    for (current=head; current->next!=first; current=current->next)
                        cost+=costMatrix[current->city][current->next->city];
                    
                    /*Aggiungo il costo dalla citt precedente first a third*/
                    cost+=costMatrix[current->city][third->city];
                    
                    /*Calcolo il costo del secondo blocco*/
                    for (current=third; current!=fourth; current=current->next)
                        cost+=costMatrix[current->city][current->next->city];

                    /*Aggiungo il costo dalla fine del secondo blocco al
                      seguito del primo*/                        
                    cost+=costMatrix[fourth->city][second->next->city];
                    
                    /*Aggiungo il costo del percorso tra i due blocchi*/
                    for (current=second->next; current->next!=third; current=current->next) 
                       cost+=costMatrix[current->city][current->next->city];
                   
                    /*Aggiungo il costo dalla citt precedente third a first*/
                    cost+=costMatrix[current->city][first->city];
                   
                    /*Aggiungo il costo del primo blocco*/
                    for (current=first; current!=second; current=current->next)
                       cost+=costMatrix[current->city][current->next->city]; 
                  
                    /*Aggiungo il costo dalla fine del primo blocco alla parte 
                      finale*/
                    cost+=costMatrix[second->city][fourth->next->city];
  
                    /*Aggiungo la parte finale*/
                    for (current=fourth->next; current!=head; current=current->next)
                        cost+=costMatrix[current->city][current->next->city];
                
                }
                
                else {
                 
                    /*Calcolo i costi fino alla citt precedente first*/
                    for (current=head; current->next!=first; current=current->next)
                        cost+=costMatrix[current->city][current->next->city];
                    
                    /*Aggiungo il costo dalla citt precedente first a third*/
                    cost+=costMatrix[current->city][third->city];
                    
                    /*Calcolo il costo del secondo blocco*/
                    for (current=third; current!=fourth; current=current->next)
                        cost+=costMatrix[current->city][current->next->city];
                    
                    /*Aggiungo il costo dalla fine del secondo blocco a first*/
                    cost+=costMatrix[fourth->city][first->city];
                   
                    /*Aggiungo il costo del primo blocco*/
                    for (current=first; current!=second; current=current->next)
                       cost+=costMatrix[current->city][current->next->city]; 
                  
                    /*Aggiungo il costo dalla fine del primo blocco alla parte 
                      finale*/
                    cost+=costMatrix[second->city][fourth->next->city];
  
                    /*Aggiungo la parte finale*/
                    for (current=fourth->next; current!=head; current=current->next)
                        cost+=costMatrix[current->city][current->next->city];                
                
                }
                
                                                
                /*Se  conveniente*/
                if (cost<lenght) {
                    lenght=cost;
                    improved=true;
                    bestFirst=first;
                    bestSecond=second;
                    bestThird=third;
                    bestFourth=fourth;
                    if (second->next==third)
                        near=true;
                    else
                        near=false;
                }
                
                     
                      
                  
            }
        
    
        }
          
                        
        /*Se ho migliorato eseguo lo scambio*/
        if (improved) {
              
            /*Se i due blocchi non sono adiacenti*/
            if (near==false) {
                
                third=bestThird->previous; 
                                                               
                bestFirst->previous->next=bestThird;
                bestThird->previous=bestFirst->previous;
            
                current=bestFourth->next;
                
                bestFourth->next=bestSecond->next;
                bestSecond->next->previous=bestFourth;
            
                third->next=bestFirst;
                bestFirst->previous=third;
            
                bestSecond->next=current;
                current->previous=bestSecond;         
            }
            
            
            else {
                       
                bestFirst->previous->next=bestThird;
                bestThird->previous=bestFirst->previous;
                
                current=bestFourth->next;
                
                bestFourth->next=bestFirst;
                bestFirst->previous=bestFourth;
                
                bestSecond->next=current;
                current->previous=bestSecond;
                        
            }
            
        }
        
    
   
    
    totalLenght=lenght;
   
}   
    


void removeCouple(void) {
    
    struct list *first;
    struct list *second=NULL; 
    struct list *pickup=NULL;
    struct list *delivery=NULL;
    struct list *current;
    unsigned int cost1, start, end, i, myStart, myEnd;
          
    unsigned int currentLenght;
    
    
    bool improved=true;
        
    improved=false;
                   
    currentLenght=totalLenght;  
         
    first=head->next;
               
    /*Cerco i vertici che se rimossi e reinseriti mi danno la maggiore
    riduzione della lunghezza del tour*/
    while (first!=head) {
                       
        /*Se  una citt PICKUP*/
        if (cities[first->city].type==PICKUP) {
                                
            /*Cerco la relativa DELIVERY*/
            second=first->next;
                
            while (cities[second->city].partner!=first->city)
               second=second->next;
              
                   
            /*Rimuovo le citt*/
            if (first->next!=second) {
                first->previous->next=first->next;
                first->next->previous=first->previous;
                second->previous->next=second->next;
                second->next->previous=second->previous;
            }
            else {
                first->previous->next=second->next;
                second->next->previous=first->previous;
            }
                        
            /*Valuto il reinserimento*/
            cost1=findBest(first->city, size-2, &start, &end, head);
                
                
            /*Se  la minore memorizzo le citt*/ 
            if (cost1<currentLenght) {
                pickup=first;
                delivery=second;
                currentLenght=cost1;
                myStart=start;
                myEnd=end;
                improved=true;
            }
                
                
            /*Reinserisco le citt*/
            if (first->next!=second) {
                first->next->previous=first;
                first->previous->next=first;
                second->previous->next=second;
                second->next->previous=second;    
            }    
            else {
                second->next->previous=second;
                first->previous->next=first;    
            }                
            
        }
            
        first=first->next;
             
    }
        
    /*Se ho migliorato tolgo e reinserisco le citt*/
    if (improved) {
        
        start=myStart;
        end=myEnd;
        
        /*Rimuovo le citt*/
        if (pickup->next!=delivery) {
            pickup->previous->next=pickup->next;
            pickup->next->previous=pickup->previous;
            delivery->previous->next=delivery->next;
            delivery->next->previous=delivery->previous;
        }
        else {
            pickup->previous->next=delivery->next;
            delivery->next->previous=pickup->previous;
        }
            
        first=pickup;
        second=delivery;
            
        /*Inserisco la PICKUP*/
        current=head;        
        for (i=0; i<start; i++) {
            current=current->next;
        }    
        
        first->next=current->next;
        (current->next)->previous=first;
        current->next=first;
        first->previous=current;

        /*Inserisco la DELIVERY*/
        end=end-start;
        
        if (end==0) {
            second->next=first->next;
            first->next->previous=second;
            first->next=second;
            second->previous=first;       
        }    
        
        else {
        
            for (i=0; i<=end; i++) {
                current=current->next;   
            }    
        
        
            second->next=current->next;
            (current->next)->previous=second;
            current->next=second;
            second->previous=current;
        }   
          
    }            
        
    current=head;
    cost1=0;
        
    for (i=0; i<=size; i++) {
        cost1+=costMatrix[current->city][current->next->city];
        current=current->next;    
    }    
        
    totalLenght=cost1;       
            
    
}

unsigned int findBest(unsigned int city, unsigned int listSize, unsigned int *myStart, unsigned int *myEnd, struct list *head) {
   
    unsigned int i,j,k,cost;
    struct list *current, *pickupStart;
    unsigned int minCost=UINT_MAX;
    unsigned int start, end;
    unsigned int pickupLoad[DIM], deliveryLoad[DIM];
    unsigned int pickupStack, deliveryStack;
   
    
    
    /*Inizializzo gli stack*/
    for (i=0; i<DIM; i++)
        pickupLoad[i]=deliveryLoad[i]=0;
        
    pickupStack=deliveryStack=0;
            
    /*Inizializzo pickupStart*/
    pickupStart=head->previous;
    
    /*Provo tutte le posizioni possibili*/
    for (i=0; i<listSize; i++) {
        
        pickupStart=pickupStart->next;
        
        /*Sistemo lo stack: se  una citt PICKUP*/
        if (cities[pickupStart->city].type==PICKUP) {
            ++pickupStack;
            pickupLoad[pickupStack]=pickupStart->city;    
        }    
        
        /*Se  una DELIVERY*/                        
        if (cities[pickupStart->city].type==DELIVERY) {
              
            --pickupStack;    
        }    
        
        /*Calcolo il costo inserendo le citt in successione*/
        cost=costMatrix[pickupStart->city][city]+costMatrix[city][cities[city].partner]\
        +costMatrix[cities[city].partner][pickupStart->next->city];
        
        if (cost<minCost) {
            minCost=cost;
            start=i;
            end=i;    
        }    
        
        /*Copio lo stack*/
        for (k=0; k<DIM; k++)
            deliveryLoad[k]=pickupLoad[k];
            
        deliveryStack=pickupStack;
        
        current=pickupStart->next;
        
        /*Provo le posizioni per la DELIVERY*/
        for (j=i+1; (j<listSize)&&(current!=head); j++) {
        
            /*Se  una citt PICKUP*/
            if (cities[current->city].type==PICKUP) {
                ++deliveryStack;
                deliveryLoad[deliveryStack]=current->city;    
            }    
            
            /*Se  una DELIVERY*/
            if (cities[current->city].type==DELIVERY) {
                            
                --deliveryStack;
            }
            
            /*Se gli stack coincidono provo il costo*/
            if (pickupLoad[pickupStack]==deliveryLoad[deliveryStack]) {
            
                cost=costMatrix[pickupStart->city][city]+costMatrix[city][pickupStart->next->city]+\
                costMatrix[current->city][cities[city].partner]+costMatrix[cities[city].partner][current->next->city];
                
                if (cost<minCost) {
                    start=i;
                    end=j;
                    minCost=cost;    
                }    
                        
            }
            
            current=current->next;       
        
        }    
        
    
    }
    
    *myStart=start;
    *myEnd=end;
   
     
    return minCost;    
       
    
    
}    
 
void removeBlock(void) {

    struct list *first, *second, *current, *pickup, *delivery;
    
    unsigned int i, j, distance, position;
    
    unsigned int currentLenght, bestLenght, newLenght;

    bool improved=false;
        
    
  
    /*Aggiorno le misure della lunghezza*/        
    bestLenght=totalLenght;  
                
    /*Per tutti i blocchi*/
    for (first=head->next; first->next!=head; first=first->next) {
                    
        /*Mi posiziono su una citt PICKUP*/
        if (cities[first->city].type!=PICKUP)
            continue;
                                    
        /*Cerco la relativa DELIVERY*/
        second=first->next;
            
        /*Setto la dimensione iniziale del blocco pari a 2 (comprende
        cio la citt PICKUP e la DELIVERY*/
        i=2;
            
        /*Nel cercare la DELIVERY incremento i che tiene conto del numero
        di vertici compreso nel blocco*/
        while (second->city!=cities[first->city].partner) {
            second=second->next;
            ++i;
        }
                       
        /*Calcolo la lunghezza del subtour contenuto nel blocco*/
        distance=0;
                                     
        for (current=first; current!=second; current=current->next)
            distance+=costMatrix[current->city][current->next->city];
       
        /*Rimuovo il blocco*/                   
        first->previous->next=second->next;
        second->next->previous=first->previous;
        
        newLenght=0;
            
        current=head;
            
        for (j=0; j<(size-i); j++) {
            newLenght+=costMatrix[current->city][current->next->city];
            current=current->next;
        }    
                              
        current=head;
                        
        /*Calcolo il costo dell'inserimento*/                                                        
        for (j=0; j<(size-i); j++) {
            currentLenght=costMatrix[current->city][first->city]+costMatrix[second->city][current->next->city]+distance+newLenght;
                                    
            /*Se il costo  il migliore*/
            if (currentLenght<bestLenght) {
                position=current->city;
                bestLenght=currentLenght;    
                improved=true;
                pickup=first;
                delivery=second;
            }    
                
            current=current->next;    
           
        }    
            
        /*Reinserisco il blocco*/
        first->previous->next=first;
        second->next->previous=second;
                                         
    }
        
    /*Se ho migliorato*/
    if (improved==true) {
        
        /*Rimuovo il blocco*/
        pickup->previous->next=delivery->next;
        delivery->next->previous=pickup->previous;
       
                                                    
        /*Scorro la lista posizionandomi sulla citt in cui inserire il blocco*/
        for (current=head; current->city!=position; current=current->next); 
            
        first=current;
        second=current->next;
            
        pickup->previous=first;
        first->next=pickup;
            
        second->previous=delivery;
        delivery->next=second;
            
        
        /*Aggiorno la lunghezza*/
        current=head;
        distance=0;
        for (j=0; j<=size; j++) {
            distance+=costMatrix[current->city][current->next->city];
            current=current->next;
        }    
            
        totalLenght=distance;
                   
    }    
     
}





          
void complexNeighbourhoodDescent(char *file) {

    unsigned int bestLenght=totalLenght;
    
    unsigned int bestTour[size];
    
    unsigned int best1, best2, best3, best4;
    
    unsigned int i;
    
    unsigned int *array;
    
    unsigned int bestTour1[size], bestTour2[size], bestTour3[size], bestTour4[size];

    struct list *current;
    
    bool improved=true;
    
    FILE *fp;
    
    
    
    bestTour[0]=0;
    
    /*Inizializzo l'array*/
    for (i=1, current=head->next; current!=head; current=current->next)
        bestTour[i++]=current->city;
    
    while(improved) {
         
        bestLenght=totalLenght;
                        
        improved=false;
                        
        /*Scambio di coppie*/
        coupleExchange();
        
        /*Salvo il valore*/
        best1=totalLenght;
        
        /*Memorizzo il tour*/
        bestTour1[0]=0;                          
        for (i=1, current=head->next; current!=head; current=current->next) 
            bestTour1[i++]=current->city;
            
        /*Ripristino il tour iniziale*/
        for (i=1, current=head->next; current!=head; current=current->next)
            current->city=bestTour[i++];
                     
       totalLenght=bestLenght;
        
        
        
        /*Scambio di blocchi*/
        blockExchange();
        
        /*Salvo il valore*/
        best2=totalLenght;
        
        /*Memorizzo il tour*/
        bestTour2[0]=0;                          
        for (i=1, current=head->next; current!=head; current=current->next) 
            bestTour2[i++]=current->city;
            
        /*Ripristino il tour iniziale*/
        for (i=1, current=head->next; current!=head; current=current->next)
            current->city=bestTour[i++];
            
        totalLenght=bestLenght;
        
        
            
        /*Rimuovo le coppie*/
        removeCouple();
        
        /*Salvo il valore*/
        best3=totalLenght;
                
        /*Memorizzo il tour*/
        bestTour3[0]=0;                          
        for (i=1, current=head->next; current!=head; current=current->next) 
            bestTour3[i++]=current->city;
            
        /*Ripristino il tour iniziale*/
        for (i=1, current=head->next; current!=head; current=current->next)
            current->city=bestTour[i++];
                   
        totalLenght=bestLenght;    
         
                       
                        
        /*Rimuovo i blocchi*/
        removeBlock();
        
        /*Salvo il valore*/
        best4=totalLenght;
                
        /*Memorizzo il tour*/
        bestTour4[0]=0;                          
        for (i=1, current=head->next; current!=head; current=current->next) 
            bestTour4[i++]=current->city;
            
        /*Ripristino il tour iniziale*/
        for (i=1, current=head->next; current!=head; current=current->next)
            current->city=bestTour[i++];
                
        totalLenght=bestLenght;
       
       
    
        /*Cerco il tour minimo*/                    
        if ((totalLenght>best1)&&(totalLenght!=best1)) {
            array=bestTour1;
            totalLenght=best1;
            improved=true;
        }
        if ((totalLenght>best2)&&(totalLenght!=best2)) {
            array=bestTour2;
            totalLenght=best2;
            improved=true;
        }
        if ((totalLenght>best3)&&(totalLenght!=best3)) {
            array=bestTour3;
            totalLenght=best3;
            improved=true;
        }
        if ((totalLenght>best4)&&(totalLenght!=best4)) {
            array=bestTour4;
            totalLenght=best4;
            improved=true;
        }    
        
        
        /*Se ho migliorato aggiorno il tour, la lunghezza e bestTour*/
        if (improved) {
           for (i=1, current=head->next; current!=head; current=current->next, i++) 
               current->city=bestTour[i]=array[i];
                            
        }
          
    
    }
    
    /*Scrivo su file il risultato*/
    if ((fp=fopen("complexNeighbourhoodDescent.txt", "a"))==NULL) {
        printf("\nCan't open output file!\n");
        exit(1);
    }    
    
    fprintf(fp, "0\n");   
             
    for (current=head->next; current!=head; current=current->next)
        fprintf(fp, "%u\n", current->city);
    
        
    fprintf(fp, "\nTotal lenght with %s is %u\n\n\n", file, totalLenght);
    
    fclose(fp);


}                
                      
                            
void variableNeighbourhoodDescent(char *file) {

    FILE *fp;
    
    struct list *current;
    
    bool improved=true;
    
    unsigned int bestTour[size], bestLenght, i;
    
    
    /*Inizializzo bestLenght e bestTour, rispettivamente lunghezza e array
    del migliore tour*/
    bestLenght=totalLenght;
    
    bestTour[0]=0;
    
    /*Inizializzo l'array*/
    for (i=1, current=head->next; current!=head; current=current->next)
        bestTour[i++]=current->city;
    
    
    /*Finch miglioro*/
    while (improved) {
       
        coupleExchange();

        /*Se ho migliorato salvo il tour*/
        if (totalLenght<bestLenght) {
            bestLenght=totalLenght;
            for (i=1, current=head->next; current!=head; current=current->next) 
                bestTour[i++]=current->city;
        }
        
        /*Altrimenti ripristino il tour iniziale*/
        else {
            for (i=1, current=head->next; current!=head; current=current->next)
                current->city=bestTour[i++];
            totalLenght=bestLenght;
        }    
    
    
        blockExchange();
        
        /*Se ho migliorato salvo il tour*/
        if (totalLenght<bestLenght) {
            bestLenght=totalLenght;
            for (i=1, current=head->next; current!=head; current=current->next) 
                bestTour[i++]=current->city;
            continue;
        }
        
        /*Altrimenti ripristino il tour iniziale*/
        else {
            for (i=1, current=head->next; current!=head; current=current->next)
                current->city=bestTour[i++];
            totalLenght=bestLenght;
        }
        
        
        removeCouple();
      
        /*Se ho migliorato salvo il tour*/
        if (totalLenght<bestLenght) {
            bestLenght=totalLenght;
            for (i=1, current=head->next; current!=head; current=current->next) 
                bestTour[i++]=current->city;
            continue;
        }
        
        /*Altrimenti ripristino il tour iniziale*/
        else {
            for (i=1, current=head->next; current!=head; current=current->next)
                current->city=bestTour[i++];
            totalLenght=bestLenght;
        }    
        
             
        removeBlock();

        /*Se ho migliorato salvo il tour*/
        if (totalLenght<bestLenght) {
            bestLenght=totalLenght;
            for (i=1, current=head->next; current!=head; current=current->next) 
                bestTour[i++]=current->city;
            continue;
        }
        
        /*Altrimenti ripristino il tour iniziale*/
        else {
            for (i=1, current=head->next; current!=head; current=current->next)
                current->city=bestTour[i++];
            totalLenght=bestLenght;
        }    
     
        
        improved=false;
        
    
    }
      
    
   
    
    /*Scrivo su file il risultato*/
    if ((fp=fopen("VariableNeighbourhoodDescent.txt", "a"))==NULL) {
        printf("\nCan't open output file!\n");
        exit(1);
    }    
    
    fprintf(fp, "0\n");   
             
    for (current=head->next; current!=head; current=current->next)
        fprintf(fp, "%u\n", current->city);
    
        
   fprintf(fp, "\nTotal lenght with %s is %u\n\n\n", file, totalLenght);
    
    fclose(fp);


}

void complexNeighbourhoodDescentHeuristic(void) {

    readTour("nn.str");
    
    complexNeighbourhoodDescent("NEAREST NEIGHBOURHOOD");
    
    readTour("rev_nn.str");
    
    complexNeighbourhoodDescent("REVERSED NEAREST NEIGHBOURHOOD");
    
    readTour("ran_nn.str");
    
    complexNeighbourhoodDescent("RANDOMIZED NEAREST NEIGHBOURHOOD");
    
    readTour("ci.str");
    
    complexNeighbourhoodDescent("CHEAPEST INSERTION");
    
    readTour("li.str");
    
    complexNeighbourhoodDescent("LARGEST INSERTION");
    
    readTour("fi.str");
    
    complexNeighbourhoodDescent("FARTHEST INSERTION");
    
    readTour("ni.str");
    
    complexNeighbourhoodDescent("NEAREST INSERTION");

    readTour("rev_ran_nn.str");
    
    complexNeighbourhoodDescent("REVERSED RANDOMIZED NEAREST NEIGHBOUR");
    
}


void vndHeuristic(void) {

    readTour("nn.str");
    
    variableNeighbourhoodDescent("NEAREST NEIGHBOURHOOD");
    
    readTour("rev_nn.str");
    
    variableNeighbourhoodDescent("REVERSED NEAREST NEIGHBOURHOOD");
    
    readTour("ran_nn.str");
    
    variableNeighbourhoodDescent("RANDOMIZED NEAREST NEIGHBOURHOOD");
    
    readTour("ci.str");
    
    variableNeighbourhoodDescent("CHEAPEST INSERTION");
    
    readTour("li.str");
    
    variableNeighbourhoodDescent("LARGEST INSERTION");
    
    readTour("fi.str");
    
    variableNeighbourhoodDescent("FARTHEST INSERTION");
    
    readTour("ni.str");
    
    variableNeighbourhoodDescent("NEAREST INSERTION");

    readTour("rev_ran_nn.str");
    
    variableNeighbourhoodDescent("REVERSED RANDOMIZED NEAREST NEIGHBOUR");
    
}


